#!/usr/bin/env python3
import os
import subprocess
import time

"""
WindowSpy
Copyright (C) 2016  Jacob Vlijm
contact: vlijm@planet.nl
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""

# paths
home = os.path.join(os.environ["HOME"])
settingspath = os.path.join(home, ".config/windowspy")
cstatepath = os.path.join(settingspath, "cycledata")
prefspath = os.path.join(settingspath, "prefs")
# files
prefs = os.path.join(prefspath, "preferences")
vport = os.path.join(cstatepath, "pinned_viewport")
window_id = os.path.join(cstatepath, "window_id")
showimage = os.path.join(cstatepath, "image.png")
# procs
show_procname = "/opt/windowspy/code/showmin"
# create paths, needed to run properly
for p in [settingspath, cstatepath, prefspath]:
    if not os.path.exists(p):
        os.mkdir(p)

def get(command):
    """
    helper function
    """
    try:
        return subprocess.check_output(command).decode("utf-8").strip()
    except subprocess.CalledProcessError:
        pass

def run(command):
    """
    helper function
    """
    try:
        subprocess.Popen(command)
    except subprocess.CalledProcessError:
        pass

def check_win():
    """
    check if the preview window exists
    """
    return get(["pgrep", "-f", show_procname])

def remove_windowdata():
    """
    remove viewport data / window- id (files)
    """
    for f in [vport, window_id]:
        try:
            os.remove(f)
        except FileNotFoundError:
            pass

def kill_preview():
    """
    kill the preview window
    """
    run(["kill", check_win()])

def get_vp():
    """
    get the current viewport
    """
    open(vport, "wt").write(get(["wmctrl", "-d"]).split()[5])
    
def three_to_one():
    """
    kill the preview, go to the original viewport and restore window properties
    if user removed the file(s) (you never know), just close the window
    """
    kill_preview()
    try:
        window = open(window_id).read().strip()
        viewport = open(vport).read().strip()
    except FileNotFoundError:
        pass
    else:
        run(["wmctrl", "-o", viewport])
        time.sleep(0.3)
        run(["wmctrl", "-i", "-r", window, "-b", "remove,sticky"])
        run(["wmctrl", "-ia", window])
        remove_windowdata()

def two_to_three(window):
    """
    read the files, run the preview window
    """
    run([show_procname, window])

def check_windowtype(wid):
    """
    check window type
    """
    check = get(["xprop", "-id", wid])
    return not any([s in check for s in [
        "_NET_WM_WINDOW_TYPE_DOCK",
        "_NET_WM_WINDOW_TYPE_DESKTOP"]])

def convert_tohex(widxd):
    """
    convert w- id xdotool -> wmctrl
    """
    return widxd[:2]+((10-len(widxd))*"0")+widxd[2:]

def edit_winprops(wid, convert=True):
    """
    minimize window, make sticky, write the id into a file
    """
    run(["xdotool", "windowminimize", wid])
    if convert:
        widxd = convert_tohex(hex(int(wid)))
    else:
        widxd = wid
    run(["wmctrl", "-i", "-r", widxd, "-b", "add,sticky"])
    get_vp()
    open(window_id, "wt").write(widxd)

def one_to_two(wlist):
    """
    get the active window, check its type. If it is the desktop or a dock,
    check for a VirtualBox window and initiate. 
    """
    wid = get(["xdotool", "getactivewindow"])
    if check_windowtype(wid):
        edit_winprops(wid)
    else:
        pidinfo = [l.split() for l in wlist.splitlines()]
        match = [l for l in pidinfo if all([
            sctools.get(["ps", "-p", l[2], "-o", "comm="]) == "VirtualBox",
            not "Manager" in l])]
        if match:
            edit_winprops(match[0][0], convert=False)
    
def get_state():
    """
    check in which of the three states we are currenrly, fix possible mismatch
    along with the job.
    """
    if check_win():
        return [3, None]
    elif os.path.exists(window_id):
        windowlist = get(["wmctrl", "-l"])
        wid = open(window_id).read().strip()
        if wid in windowlist:
            return [2, wid]
        else:
            remove_windowdata()
            return [1, None]
    else:
        return [1, None]

def get_screenres():
    win = get("xrandr").split()
    return [int(n) for n in win[
        win.index("primary")+1
        ].split("+")[0].split("x")]

def read_settings(setting):
    subj = os.path.join(prefspath, setting)
    try:
        return open(subj).read().strip()
    except FileNotFoundError:
        pass

def write_settings(setting, newval):
    subj = os.path.join(prefspath, setting)
    open(subj, "wt").write(newval)

def get_current():
    # read the current color setings. "white" if not set
    fallback = [65535, 65535, 65535]
    try:
        return [int(s) for s in read_settings("bcolor").split()]
    except AttributeError:
        return fallback
    except ValueError:
        # repeated, but wontfix
        os.remove(os.path.join(prefspath, "bcolor"))
        return fallback
        
        
        
    



    
